/* Our windowsnapper class */
objectdef windowsnapper
{
	/* Late initialization method, as opposed to using the object constructor. We want to  
	   create the object before we initialize it.
        */
	method Go()
	{
		/* Registering events before we attach ensures that the event exists, to attach to */
		LavishScript:RegisterEvent[On Deactivate]
		LavishScript:RegisterEvent[On Activate]

		/* Attach our methods to the events we registered for */
		Event[On Deactivate]:AttachAtom[This:OnDeactivate]
		Event[On Activate]:AttachAtom[This:OnActivate]

		/* Spit out startup spam */
		echo \agWindowSnapper: Window will snap to ${SnapPos} with size ${SnapSize}
		echo To change snap position, enter command (e.g.) \aysnappos 123,456
		echo To change snap size, enter command (e.g.) \aysnapsize 640x480
		echo WindowSnapper also accepts -size 640x480 and -pos 123,456 as parameters, such as
		echo \ayrun windowsnapper -size 640x480 -pos 123,456
	}

	/* When Store is called, we populate these */
	variable int X
	variable int Y
	variable int Width
	variable int Height
	variable string Frame

	/* This tracks whether we're the active window or not. Without it, window size and position could
           get messy. */
	variable bool Active=TRUE

	variable bool Ignore=FALSE

	/* This is our snap-to size and position. When not the active window, we use these. */
	variable string SnapSize=320x240
	variable string SnapPos=800,0

	/* Store the current display position and size for later use */
	method Store()
	{
		X:Set[${Display.Window.X}]
		Y:Set[${Display.Window.Y}]
		Width:Set[${Display.Width}]
		Height:Set[${Display.Height}]
		Frame:Set["${Display.Window.Frame}"]
	}

	/* Restore the stored display position and size */
	method Restore()
	{
		squelch windowpos -stealth ${X},${Y}
		squelch windowsize -rescale -viewable ${Width}x${Height}
		windowframe -stealth ${Display.Window.Frame}
	}

	/* Handler for the "On Deactivate" event, which fires when the window is no longer the active one */
	method OnDeactivate()
	{
		if ${Ignore}
			return
		if ${Active}
		{
			This:Store
			windowframe -stealth none
			squelch windowsize -rescale -viewable ${SnapSize}
			squelch windowpos -stealth ${SnapPos}
			windowvisibility -noactivate alwaysontop
			Active:Set[FALSE]
		}
	}
	
	/* Handler for the "On Activate" event, which fires when the window is once again the active one */
	method OnActivate()
	{
		Ignore:Set[FALSE]
		if !${Active}
		{
			This:Restore
			windowvisibility foreground
			Active:Set[TRUE]
		}
	}
	
}

/* Declare our windowsnapper object, called Snapper */
variable windowsnapper Snapper

/* Create a command called snapsize to change the size. The command is here until the script is terminated. */
atom(global) snapsize(string size)
{
	Snapper.SnapSize:Set["${size}"]
		echo \agWindowSnapper: Window will snap to ${Snapper.SnapPos} with size ${Snapper.SnapSize}
}

/* Create a command called snappos to change the position. The command is here until the script is terminated. */
atom(global) snappos(string position)
{
	Snapper.SnapPos:Set["${position}"]
		echo \agWindowSnapper: Window will snap to ${Snapper.SnapPos} with size ${Snapper.SnapSize}
}

/* Create a command called snapignore to ignore the next deactivation, until reactivated (again to change your mind) */
atom(global) snapignore()
{
	if ${Snapper.Ignore:Toggle}
		echo \agWindowSnapper: Ignoring next deactivation
	else
		echo \agWindowSnapper: NOT Ignoring next deactivation
}

/* Entry point for the script. All parameters passed to it will be put in an array called Params */
function main(... Params)
{  
	/* Process parameters */
	variable int p=1
	for (${p}<=${Params.Size} ; p:Inc)
	{
		/* Switch does a string comparison, so parameter processing is easy */
		switch ${Params[${p}]}
		{
		case -size
			/* we got the -size switch, so if there's another parameter, set Snapper.SnapSize to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				Snapper.SnapSize:Set["${Params[${p}]}"]
			break
		case -pos
			/* we got the -pos switch, so if there's another parameter, set Snapper.SnapPos to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				Snapper.SnapPos:Set["${Params[${p}]}"]
			break
		}
	}

	/* Create a bind for snapignore toggling */
	bind snapignore CTRL+ALT+P snapignore
	
	/* Parameters have been processed, let's get going. */	 
	Snapper:Go
	
	/* Now just keep the script running, so our functionality doesn't go away */
	while 1
	{
		waitframe
	}
}

atom atexit()
{
	bind -delete snapignore
}
